unit Unit22;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.TMSFNCTypes, FMX.TMSFNCUtils, FMX.TMSFNCGraphics, FMX.TMSFNCGraphicsTypes,
  FMX.TMSFNCCustomControl, FMX.TMSFNCWebBrowser, FMX.TMSFNCMaps,
  FMX.TMSFNCGoogleMaps, FMX.TMSFNCMapsCommonTypes, Generics.Collections, FMX.TMSFNCPersistence;

type
  TCoordinateList = class(TObjectList<TTMSFNCMapsCoordinateItem>, ITMSFNCBaseListIO, ITMSFNCBasePersistenceIO)
  private
    FOwnerInterface: IInterface;
  protected
    function GetItemClass: TClass;
    function CreateObject(const AClassName: string; const ABaseClass: TClass): TObject;

    { IInterface }
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
  public
    function QueryInterface(const IID: TGUID; out Obj): HResult; virtual; stdcall;
  end;

  TForm22 = class(TForm)
    TMSFNCGoogleMaps1: TTMSFNCGoogleMaps;
    procedure FormCreate(Sender: TObject);
    procedure TMSFNCGoogleMaps1PolyElementEditEnd(Sender: TObject;
      AEventData: TTMSFNCMapsEventData);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form22: TForm22;

implementation

{$R *.fmx}

procedure TForm22.FormCreate(Sender: TObject);
var
  ar: TTMSFNCMapsCoordinateRecArray;
  p: TTMSFNCGoogleMapsPolyline;
begin
  SetLength(ar, 2);

  TMSFNCGoogleMaps1.APIKey := ;

  ar[0].Latitude := TMSFNCGoogleMaps1.Options.DefaultLatitude;
  ar[0].Longitude := TMSFNCGoogleMaps1.Options.DefaultLongitude;

  ar[1].Latitude := TMSFNCGoogleMaps1.Options.DefaultLatitude + 0.05;
  ar[1].Longitude := TMSFNCGoogleMaps1.Options.DefaultLongitude + 0.05;

  TMSFNCGoogleMaps1.BeginUpdate;
  p := TTMSFNCGoogleMapsPolyline(TMSFNCGoogleMaps1.AddPolyline(ar));
  p.Editable := True;
  TMSFNCGoogleMaps1.EndUpdate;
end;

procedure TForm22.TMSFNCGoogleMaps1PolyElementEditEnd(Sender: TObject;
  AEventData: TTMSFNCMapsEventData);
var
  p: TTMSFNCGoogleMapsPolyline;
  a: TCoordinateList;
  I: Integer;
begin
  if Assigned(AEventData.PolyElement) then
  begin
    if (AEventData.PolyElement is TTMSFNCGoogleMapsPolyline) then
    begin
      p := (AEventData.PolyElement as TTMSFNCGoogleMapsPolyline);
      a := TCoordinateList.Create;
      a.JSON := AEventData.CustomData;

      TMSFNCGoogleMaps1.BeginUpdate;

      try
        p.Recreate := True;
        p.Coordinates.Clear;
        for I := 0 to a.Count - 1 do
          p.Coordinates.Add.Assign(a[I]);

        TTMSFNCUtils.Log(p.Coordinates.JSON);
      finally
        a.Free;
      end;

      TMSFNCGoogleMaps1.EndUpdate;
    end;
  end;
end;

{ TCoordinateList }


function TCoordinateList.CreateObject(const AClassName: string;
  const ABaseClass: TClass): TObject;
begin
  Result := TTMSFNCMapsCoordinateItem.Create(nil);
end;

function TCoordinateList.GetItemClass: TClass;
begin
  Result := TTMSFNCMapsCoordinateItem;
end;


function TCoordinateList._AddRef: Integer;
begin
  if FOwnerInterface <> nil then
    Result := FOwnerInterface._AddRef else
    Result := -1;
end;


function TCoordinateList._Release: Integer;
begin
  if FOwnerInterface <> nil then
    Result := FOwnerInterface._Release else
    Result := -1;
end;


function TCoordinateList.QueryInterface(const IID: TGUID;
  out Obj): HResult;
const
  E_NOINTERFACE = HResult($80004002);
begin
  if GetInterface(IID, Obj) then Result := 0 else Result := E_NOINTERFACE;
end;

end.
